#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import os
from setuptools import setup


def check_pyversions(versions):
    """checks if the python-version is within the list of versions

    to check is we are running py2 or py3:
        check_pyversions(((2,), (3,)))

    to check if we are running either py2 or py3.4.2 (but no other py3 version)
        check_pyversions(((2,), (3,4,2)))
    """
    for v in versions:
        if all([x == y for x, y in zip(v, sys.version_info)]):
            return True
    return False


# ModuleFinder can't handle runtime changes to __path__, but win32com uses them
try:
    # py2exe 0.6.4 introduced a replacement modulefinder.
    # This means we have to add package paths there, not to the built-in
    # one.  If this new modulefinder gets integrated into Python, then
    # we might be able to revert this some day.
    # if this doesn't work, try import modulefinder
    try:
        import py2exe.mf as modulefinder
    except ImportError:
        import modulefinder
    import win32com

    for p in win32com.__path__[1:]:
        modulefinder.AddPackagePath("win32com", p)
    for extra in ["win32com.shell"]:  # ,"win32com.mapi"
        __import__(extra)
        m = sys.modules[extra]
        for p in m.__path__[1:]:
            modulefinder.AddPackagePath(extra, p)
    import py2exe
except ImportError:
    # no build path setup, no worries.
    pass


# This is a list of files to install, and where
# (relative to the 'root' dir, where setup.py is)
# You could be more specific.
data_files = [
    "deken.hy",
]
setup_requires = []
dist_dir = ""
dist_file = None

options = {}
setupargs = {
    "name": "deken",
#    "version": "0.2.4",
    "description": """Pure Data externals wrangler""",
    "author": "Chris McCormick, IOhannes m zmölnig et al.",
    "author_email": "pd-list@lists.puredata.info",
    "url": "https://git.iem.at/pure-data/deken",
    # Name the folder where your packages live:
    # (If you have other packages (dirs) or modules (py files) then
    # put them into the package directory - they will be found
    # recursively.)
    'packages': ['deken', ],
    'package_dir': {'deken': 'src/deken'},
    'package_data': {'deken': data_files},
    "install_requires": [
        # 'PySide',
    ],
    "long_description": """
a tool to create and upload dek packages to puredata.info,
so they can be installed by Pd's built-in package manager
""",
    #
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    "classifiers": [
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD 3 clause",
        "Natural Language :: English",
        # "Topic :: Internet :: WWW/HTTP :: Site Management",
    ],
    "options": options,
    "setup_requires": setup_requires,
}

if "py2exe" in sys.argv:
    dist_dir = "%s-%s" % (setupargs["name"], setupargs["version"])
    dist_file = "%s.exe-%s.zip" % (setupargs["name"], setupargs["version"])
    setup_requires += [
        "py2exe",
    ]

    def getMSVCfiles():
        # urgh, find the msvcrt redistributable DLLs
        # either it's in the MSVC90 application folder
        # or in some winsxs folder
        from glob import glob

        program_path = os.path.expandvars("%ProgramFiles%")
        winsxs_path = os.path.expandvars("%SystemRoot%\WinSXS")
        msvcrt_paths = [
            (
                r"%s\Microsoft Visual Studio 9.0\VC\redist\x86\Microsoft.VC90.CRT"
                % program_path
            )
        ]

        if check_pyversions(((2, 7),)):
            # python2.7 seems to be built against VC90 (9.0.21022),
            # so let's try that
            msvcrt_paths += glob(
                r"%s\x86_microsoft.vc90.crt_*_9.0.21022.8_*_*" "\\" % winsxs_path
            )
            for p in msvcrt_paths:
                if os.path.exists(os.path.join(p, "msvcp90.dll")):
                    sys.path.append(p)
                    f = glob(r"%s\*.*" % p)
                    if f:
                        return [("Microsoft.VC90.CRT", f)]
                    return None

        return None

    def getRequestsCerts():
        import requests.certs

        f = requests.certs.where()
        if f:
            return [(".", [f])]

    data_files += getMSVCfiles() or []
    data_files += getRequestsCerts() or []
    print(data_files)

    setupargs["windows"] = [
        {
            # 'icon_resources': [(1, "media\deken.ico")],
            "script": "pydeken.py",
        }
    ]
    setupargs["zipfile"] = None

    options["py2exe"] = {
        "includes": [
            "deken.hy",
        ],
        "packages": [
            "requests",
        ],
        "bundle_files": 3,
        "dist_dir": os.path.join("dist", dist_dir),
    }

if "py2app" in sys.argv:
    dist_dir = "%s.app" % (setupargs["name"],)
    dist_file = "%s.app-%s.zip" % (setupargs["name"], setupargs["version"])
    setup_requires += [
        "py2app",
    ]
    setupargs["app"] = [
        "pydeken.py",
    ]
    options["py2app"] = {
        "packages": [
            "requests",
        ],
    }

if dist_dir:
    try:
        os.makedirs(os.path.join("dist", dist_dir))
    except FileExistsError:
        pass

setup(**setupargs)

if dist_dir and dist_file:
    import zipfile

    def zipdir(path, ziph):
        # ziph is zipfile handle
        for root, dirs, files in os.walk(path):
            for f in files:
                fullname = os.path.join(root, f)
                archname = os.path.relpath(fullname, os.path.join(path, ".."))
                if ziph:
                    ziph.write(fullname, archname)
                else:
                    print("%s -> %s" % (fullname, archname))

    with zipfile.ZipFile(dist_file, "w", compression=zipfile.ZIP_DEFLATED) as myzip:
        zipdir(os.path.join("dist", dist_dir), myzip)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This software is copyrighted by Chris McCormick and others.  The following
# terms (the "Standard Improved BSD License") apply to all files associated
# with the software unless explicitly disclaimed in individual files:
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
# 3. The name of the author may not be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.

# this is a thin wrapper around deken.hy, to launch it without a
# HY executable (or file associations)

try:
    import argparse
    import copy
    import getpass
    import os
    import re
    import sys
    import tarfile
    import zipfile

    import datetime

    import elftools
    import macholib
    import pefile

    import gnupg
    import hashlib

    import requests

    import subprocess

except ImportError:
    pass

# 'keyring' is disabled here as it makes problems with pyinstaller
try:
    import keyring.backends.Windows
    import keyring

    pass
except ImportError:
    pass

try:
    import webbrowser
except ImportError:
    pass

try:
    import easywebdav2
except ImportError:
    try:
        import easywebdav
    except ImportError:
        pass

try:
    import ConfigParser
    import StringIO
    import urlparse
except ImportError:
    import configparser
    import io
    import urllib.parse


import hy
deken = None

if not deken:
    try:
        import deken.deken
        deken = deken.deken
    except ImportError:
        pass
if not deken:
    try:
        from . import deken
    except ImportError:
        pass
if not deken:
    try:
        import deken
    except ImportError:
        pass

## on macOS, pyinstaller requires more help...
try:
    import runpy
    import hy.core.bootstrap
    import hy.contrib.loop
except ImportError:
    pass


def askpass(prompt="Password: ", fallback=None):
    try:
        subprocess.call(["stty", "-echo"])
    except:
        if fallback:
            return fallback(prompt)
        return None
    sys.stdout.write(prompt)
    sys.stdout.flush()
    password = None
    try:
        try:
            password = raw_input()
        except NameError:
            password = input()
    except:
        pass
    sys.stdout.write("\n")
    sys.stdout.flush()
    subprocess.call(["stty", "echo"])
    return password


def resource_path(relative_path):
    """Get absolute path to resource, works for dev and for PyInstaller"""
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
    except Exception:
        base_path = os.path.abspath(".")
    return os.path.join(base_path, relative_path)


def _main():
    _version = None

    if _version is None:
        try:
            from ._version import version as _version
        except ImportError:
            pass

    if _version is None:
        try:
            with open(resource_path("DEKEN_VERSION"), "r") as f:
                _version = f.read().strip()
        except OSError:
            pass
        except IOError:
            pass

    if _version is not None:
        deken.version = _version

    deken_askpass = deken.askpass
    deken.askpass = lambda prompt: askpass(prompt, deken_askpass)
    deken.main()


if __name__ == "__main__":
    _main()



#!/usr/bin/env python3

import re
import fnmatch
import itertools

aptcache = None


class EmptyVersion:
    """helper class that provides an empty versions member"""

    def __init__(self, versions=[]):
        self.versions = versions


def parseArgs():
    import argparse

    parser = argparse.ArgumentParser()

    p = parser.add_argument_group(
        title="constraints",
    )

    p.add_argument(
        "--os",
        default="Linux",
        choices=["Linux"],
        help="target operating system [DEFAULT: %(default)r]",
    )
    p.add_argument(
        "--architecture",
        default=None,
        help="target CPU architecture [DEFAULT: 'native']",
    )
    p.add_argument(
        "--floatsize",
        type=int,
        default=32,
        help="target floatsize [DEFAULT: %(default)s}]",
    )

    p = parser.add_argument_group(
        title="API",
    )
    p.add_argument(
        "--api",
        type=int,
        choices=[
            0,
            1,
        ],
        required=True,
        help="Output format version (0=test; 1=current)",
    )

    parser.add_argument(
        "pkg",
        nargs="*",
        help="package(s) to search for",
    )

    args = parser.parse_args()

    return args


def initializeAptCache():
    import apt

    global aptcache
    aptcache = apt.Cache()


def stripSuffix(s, suffix):
    if s.startswith(suffix):
        return s[len(suffix) :]
    return s


def hasDependency(versioned_pkg, dependencies: list) -> bool:
    """returns True iff <versioned_pkg> (apt.package.Version) depends
    on any package in <dependencies> (list[str]);
    otherwise returns False"""
    for d in versioned_pkg.dependencies:
        for bd in d:
            if bd.name in dependencies:
                return True
    return False


def getPackages(pkgs, arch=None, floatsize=32):
    pd32deps = {"puredata-core", "puredata", "puredata-gui", "pd"}
    pd64deps = {"puredata64-core", "puredata64", "puredata-gui", "pd64"}
    if floatsize == 64:
        pddeps = pd64deps
    else:
        pddeps = pd32deps

    if pkgs == ["*"]:
        pkgs = []

    pkgmatch = re.compile(
        "(pd-|pd64-)?(" + "|".join(fnmatch.translate(p) for p in pkgs) + ")"
    ).match

    packages = {}
    for p in aptcache:
        # filter out known non-externals
        if p.shortname.startswith("puredata"):
            continue
        # filter out unwanted architectures
        if arch and p.architecture() != arch:
            continue
        for v in p.versions:
            if pkgs:
                names = [p.shortname] + v.provides
                matches = [
                    matchedname
                    for name in {*names}
                    for matchedname in [pkgmatch(name)]
                    if matchedname
                ]
                if not matches:
                    continue
            else:
                # all externals
                matches = [p.shortname, None, None]
                if p.shortname.startswith("pd-"):
                    matches[2] = p.shortname[3:]
                elif p.shortname.startswith("pd64-"):
                    matches[2] = p.shortname[5:]
                matches = [matches]

            # we only take packages that depend on Pd
            if not hasDependency(v, pddeps):
                continue
            for m in matches:
                for n in {m[0], m[2]}:
                    npkgs = packages.get(n) or set()
                    npkgs.add(v)
                    packages[n] = npkgs
    return itertools.chain(*packages.values())


def getOrigin(
    versioned_pkg,
    fallback_origin="apt",
    fallback_date=None,
    trusted="",
    untrusted="?",
):
    """get the (one) package origin as a tuple of repository & archive,
    e.g. ("Debian", "bookworm/main")
    if the repository is trusted, the <trusted> string is appended, otherwise the <untrusted>.
    prefer trusted sources over untrusted.
    """
    origins = [o for o in versioned_pkg.origins if o.trusted]
    trust = trusted
    if not origins:
        origins = versioned_pkg.origins
        trust = untrusted

    if not origins:
        return (fallback_origin, fallback_date, None)

    origin = None
    codename = None
    component = None
    for o in origins:
        if all((origin, codename, component)):
            break
        try:
            origin = origin or o.label or o.origin or fallback_origin
        except AttributeError:
            pass
        try:
            codename = codename or o.codename or o.archive
        except AttributeError:
            pass
        try:
            component = component or o.component
        except AttributeError:
            pass

    origin = origin or fallback_origin
    if codename or component:
        if not component:
            codename_component = codename
        else:
            codename_component = f"{codename or '???'}/{component}"
    else:
        codename_component = fallback_date

    return (origin, codename_component)


def showPackages(pkgs):
    """create a parseseable representation for each package"""
    for p in sorted(set(pkgs)):
        library = p.package.name
        version = p.version
        arch = p.architecture
        uploader, date = getOrigin(p)
        uri = p.uri
        status = p.summary
        installed = p.package.is_installed
        state = "Already installed" if installed else "Provided"
        comment = f"{state} by {uploader} ({date})"

        print(
            f"{library}\t{version}\t{arch}\t{int(installed)}\t{uploader}\t{date}\t{uri}\t{status}\t{comment}"
        )


def main():
    args = parseArgs()
    if args.api not in {0, 1}:
        return
    if not args.api:
        # quick sanity check if python-apt is available
        import apt

        return

    initializeAptCache()
    if not args.architecture:
        for p in ["dpkg", "apt", "bash"]:
            try:
                args.architecture = aptcache.get(f"{p}:native").architecture()
                break
            except AttributeError:
                pass

    packages = getPackages(args.pkg, arch=args.architecture, floatsize=args.floatsize)
    showPackages(packages)


if __name__ == "__main__":
    main()



# -*- mode: python -*-

#-----------------------------------------------------------------------------
# Copyright (c) 2017, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License with exception
# for distributing bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

# Hook for the macholib module: https://bitbucket.org/ronaldoussoren/macholib

hiddenimports = ['macholib.MachO', 'macholib.SymbolTable']
